@@ -1,6 +1,7 @@
 """EnergySeries module."""
 
 import copy
+import logging
 import warnings
 from datetime import timedelta
 
@@ -13,8 +14,10 @@ from pandas import (
     MultiIndex,
     Series,
     date_range,
+    infer_freq,
     pivot_table,
     to_datetime,
+    to_timedelta,
 )
 from pandas._libs.tslibs.offsets import to_offset
 from pandas.core.generic import NDFrame
@@ -29,9 +32,7 @@ from .plotting import (
     _setup_subplots,
     save_and_show,
 )
-
-import logging
-from .units import unit_registry
+from .units import unit_registry, IP_DEFAULT_CONVERSION, SI_DEFAULT_CONVERSION
 
 log = logging.getLogger(__name__)
 
@@ -109,7 +110,8 @@ class EnergySeries(Series):
         )
         self.bin_edges_ = None
         self.bin_scaling_factors_ = None
-        self.units = units
+
+        self._set_units(units)
 
         for k, v in kwargs.items():
             EnergySeries._metadata.append(k)
@@ -124,7 +126,7 @@ class EnergySeries(Series):
             for name in other._metadata:
                 if name == "units":
                     if isinstance(other, EnergyDataFrame):
-                        setattr(self, name, getattr(other, "_units").get(self.name))
+                        setattr(self, name, getattr(other, "units").get(self.name))
                     else:
                         setattr(self, name, getattr(other, "units"))
                 elif name == "name":
@@ -199,18 +201,14 @@ class EnergySeries(Series):
         es.index = newindex
         return es
 
-    @property
-    def units(self) -> dict or Unit:
-        return self._units
-
-    @units.setter
-    def units(self, value):
+    def _set_units(self, value):
+        """Set units on self."""
         if isinstance(value, str):
-            self._units = unit_registry.parse_expression(value).units
+            self.units = unit_registry.parse_expression(value).units
         elif isinstance(value, (Unit, Quantity)):
-            self._units = value
+            self.units = value
         elif value is None:
-            self._units = unit_registry.parse_expression(value).units
+            self.units = unit_registry.parse_expression(value).units
         else:
             raise TypeError(f"Unit of type {type(value)}")
 
@@ -298,18 +296,18 @@ class EnergySeries(Series):
 
         Args:
             to_units (str, pint.Unit):
-            inplace:
+            inplace (bool): If True, conversion is applied inplace.
         """
-        cdata = unit_registry.Quantity(self.values, self.units).to(to_units).m
+        cdata = unit_registry.Quantity(self.values, self.units).to(to_units)
         if inplace:
-            self[:] = cdata
-            self.units = to_units
+            self[:] = cdata.m
+            self._set_units(cdata.units)
         else:
             # create new instance using constructor
-            result = self._constructor(data=cdata, index=self.index, copy=False)
+            result = self._constructor(data=cdata.m, index=self.index, copy=False)
             # Copy metadata over
             result.__finalize__(self)
-            result.units = to_units
+            result._set_units(to_units)
             return result
 
     def normalize(self, inplace=False):
@@ -446,8 +444,8 @@ class EnergySeries(Series):
         """
         if energy_series.empty:
             warnings.warn(
-                "The EnergyProgile you are attempting to plot is "
-                "empty. Nothing has been displayed.",
+                "The EnergySeries you are attempting to plot is empty. "
+                "Nothing has been displayed.",
                 UserWarning,
             )
             return axes
@@ -617,6 +615,7 @@ class EnergySeries(Series):
 
     @property
     def time_at_min(self):
+        """Return the index value where the min occurs."""
         return self.idxmin()
 
     @property
@@ -639,16 +638,42 @@ class EnergySeries(Series):
         else:
             return self.data.shape[1]
 
+    def to_si(self, inplace=False):
+        """Convert self to SI units.
+
+        Args:
+            inplace (bool): If True, conversion is applied inplace.
+        """
+        try:
+            si_units = SI_DEFAULT_CONVERSION[self.units]
+        except KeyError:
+            return self
+        self.to_units(si_units, inplace=inplace)
+        return self
+
+    def to_ip(self, inplace=False):
+        """Convert self to IP units (inch-pound).
+
+        Args:
+            inplace (bool): If True, conversion is applied inplace.
+        """
+        try:
+            ip_units = IP_DEFAULT_CONVERSION[self.units]
+        except KeyError:
+            return self
+        self.to_units(ip_units, inplace=inplace)
+        return self
+
     def plot2d(
         self,
-        periodlength=24,
+        periodlength=None,
         vmin=None,
         vmax=None,
         vcenter=None,
         axis_off=True,
         cmap="RdBu",
         fig_height=None,
-        fig_width=6,
+        fig_width=8,
         show=True,
         save=False,
         close=False,
@@ -658,6 +683,8 @@ class EnergySeries(Series):
         ax=None,
         filename="untitled",
         extent="tight",
+        ylabel=None,
+        xlabel=None,
     ):
         """
         Args:
@@ -682,6 +709,8 @@ class EnergySeries(Series):
             ax:
             filename:
             extent:
+            ylabel (str): Set the label for the y-axis.
+            xlabel (str): Set the label for the x-axis.
         """
         if fig_height is None:
             fig_height = fig_width / 3
@@ -698,14 +727,34 @@ class EnergySeries(Series):
                 fig.set_size_inches(figsize)
             axes = ax
 
-        stacked, timeindex = tsam.unstackToPeriods(copy.deepcopy(self), periodlength)
+        freq = infer_freq(self.index[0:3])  # infer on first 3.
+        offset = to_offset(freq)
+
+        if periodlength is None:
+            periodlength = 24 * 1 / (to_timedelta(offset).seconds / 3600)
+
+        if ylabel is None:
+            yperiod = (periodlength * offset).delta
+
+            offset_n = f"{offset.n}-" if offset.n > 1 else ""
+            ylabel = (
+                f"{offset_n}{RESOLUTION_NAME[offset.name]} of "
+                f"{RESOLUTION_NAME[yperiod.resolution_string][0:-1]}"
+            )
+
+        stacked, timeindex = tsam.unstackToPeriods(
+            copy.deepcopy(self), int(periodlength)
+        )
+        if xlabel is None:
+            xperiod = (periodlength * offset).delta
+            xlabel = f"{RESOLUTION_NAME[xperiod.resolution_string]}"
         cmap = plt.get_cmap(cmap)
         if vcenter is not None:
             norm = TwoSlopeNorm(vcenter, vmin=vmin, vmax=vmax)
         else:
             norm = None
         im = axes.imshow(
-            stacked.values.T,
+            stacked.T,
             interpolation="nearest",
             vmin=vmin,
             vmax=vmax,
@@ -713,9 +762,12 @@ class EnergySeries(Series):
             norm=norm,
         )
         axes.set_aspect("auto")
-        axes.set_ylabel("Hour of day")
-        axes.set_xlabel("Day of year")
-        axes.set_title(f"{self.name}")
+        axes.set_ylabel(ylabel)
+        axes.set_xlabel(xlabel)
+
+        ax_title = f"{self.name}" if self.name is not None else None
+        if ax_title:
+            axes.set_title(ax_title)
 
         # fig.subplots_adjust(right=1.1)
         cbar = fig.colorbar(im, ax=axes)
@@ -728,6 +780,17 @@ class EnergySeries(Series):
         return fig, axes
 
 
+RESOLUTION_NAME = dict(
+    D="Days",
+    H="Hours",
+    T="Minutes",
+    S="Seconds",
+    L="Milliseconds",
+    U="Microseconds",
+    N="Nanoseconds",
+)
+
+
 class EnergyDataFrame(DataFrame):
     """EnergyDataFrame class.
 
@@ -736,7 +799,7 @@ class EnergyDataFrame(DataFrame):
 
     Data structure also contains labeled axes (rows and columns). Arithmetic
     operations align on both row and column labels. Can be thought of as a
-    dict-like container for Series objects. The primary energypandas data structure.
+    dict-like container for Series objects. The primary energy-pandas data structure.
 
     """
 
@@ -775,14 +838,14 @@ class EnergyDataFrame(DataFrame):
         )
 
         # prepare units dict; holds series units.
-        self._units = {}
+        self.units = {}
         if isinstance(data, (dict, DataFrame)):
             # for each series, setattr `units` defined in each items or for whole df.
             for name, col in data.items():
                 # ndarray (structured or homogeneous), Iterable, dict, or DataFrame
-                self._units[name] = getattr(col, "units", self._parse_units(units))
+                self.units[name] = getattr(col, "units", self._parse_units(units))
         elif isinstance(data, EnergySeries):
-            self._units[data.name] = data.units
+            self.units[data.name] = data.units
 
         # for each extra kwargs, set as metadata of self.
         for k, v in kwargs.items():
@@ -867,14 +930,6 @@ class EnergyDataFrame(DataFrame):
             edf.sort_values(sort_values, inplace=True)
         return edf
 
-    @property
-    def units(self):
-        return self._units
-
-    @units.setter
-    def units(self, value):
-        self._units = value
-
     def _parse_units(self, value):
         if isinstance(value, str):
             return unit_registry.parse_expression(value).units
@@ -886,28 +941,41 @@ class EnergyDataFrame(DataFrame):
             raise TypeError(f"Unit of type {type(value)}")
 
     def to_units(self, to_units=None, inplace=False):
-        """returns the multiplier to convert units
+        """Returns the multiplier to convert units.
 
         Args:
             to_units (str or pint.Unit):
-            inplace:
+            inplace (bool): If True, conversion is applied inplace.
+
+        Examples:
+            >>> import energy_pandas as epd
+            >>> edf = epd .EnergyDataFrame(
+            >>>         {
+            >>>             "Series 1": epd.EnergySeries(range(0, 8760), units="degC"),
+            >>>             "Series 2": epd.EnergySeries(range(0, 8760), units="degK"),
+            >>>             "Series 3": epd.EnergySeries(range(0, 8760), units="degR"),
+            >>>         },
+            >>> )
+            >>> edf.to_units("degC").units
+            {'Series 1': <Unit('degree_Celsius')>, 'Series 2': <Unit('degree_Celsius')>, 'Series 3': <Unit('degree_Celsius')>}
+
         """
-        if len(set(self.units.values())) > 1:
-            raise MultipleUnitsError
-        else:
-            units = next(iter(set(self.units.values())))
-        cdata = unit_registry.Quantity(self.values, units).to(to_units)
+        cdata = self.apply(
+            lambda col: unit_registry.Quantity(col.values, col.units).to(to_units)
+        )
         if inplace:
-            self[:] = cdata.m
-            self._units = {name: cdata.units for name in self.columns}
+            self[:] = cdata.values
+            self.units = {col: u for col, u in zip(self.columns, cdata.units.values())}
         else:
             # create new instance using constructor
             result = self._constructor(
-                data=cdata.m, index=self.index, columns=self.columns, copy=False
+                data=cdata, index=self.index, columns=self.columns, copy=False
             )
             # Copy metadata over
             result.__finalize__(self)
-            result._units = {name: cdata.units for name in self.columns}
+            result.units = {
+                col: u for col, u in zip(self.columns, cdata.units.values())
+            }
             return result
 
     def normalize(self, inplace=False):
@@ -918,7 +986,7 @@ class EnergyDataFrame(DataFrame):
             # replace whole data with array
             self[:] = x_scaled
             # change units to dimensionless
-            self._units = {name: unit_registry.dimensionless for name in self.columns}
+            self.units = {name: unit_registry.dimensionless for name in self.columns}
         else:
             # create new instance using constructor
             result = self._constructor(
@@ -995,11 +1063,6 @@ class EnergyDataFrame(DataFrame):
             subplots, nseries, sharex, sharey, figsize, ax, layout, layout_type
         )
         cols = self.columns
-        if periodlength is None:
-            import pandas as pd
-
-            freq = pd.infer_freq(self.index[0:3])  # infer on first 3.
-            periodlength = 24 * 1 / (pd.to_timedelta(to_offset(freq)).seconds / 3600)
         for ax, col in zip(axes, cols):
             self[col].plot2d(
                 periodlength=periodlength,
